#!/opt/concourse/worker/volumes/live/f42d4aad-1b9c-446f-441c-1fabebea4417/volume/anaconda-navigator_1648477375444/_h_env_placehold_placehold_placehold_placehold_placehold_placehold_placehold_placehold_placehold_placehold_placehold_placehold_placehold_placeh/Anaconda-Navigator.app/Contents/MacOS/python
from subprocess import call

call(["/opt/concourse/worker/volumes/live/f42d4aad-1b9c-446f-441c-1fabebea4417/volume/anaconda-navigator_1648477375444/_h_env_placehold_placehold_placehold_placehold_placehold_placehold_placehold_placehold_placehold_placehold_placehold_placehold_placehold_placeh/bin/anaconda-navigator"])
